/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Download Statusbar.
 *
 * The Initial Developer of the Original Code is
 * Devon Jensen.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Devon Jensen <velcrospud@hotmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


const db_kObserverServiceProgID = "@mozilla.org/observer-service;1";
const db_NC_NS = "http://home.netscape.com/NC-rdf#";

var db_gDownloadManager;
var db_currentelem;
//var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);

const db_dlObserver = {
  observe: function(subject, topic, state) {
  	
  	//aConsoleService.logStringMessage(topic + "  -  " + state);
  	
  	var db_dl = subject.QueryInterface(Components.interfaces.nsIDownload);
  	var elmpath = db_dl.target.path
	var db_elt = document.getElementById(elmpath);
  	
    if (topic == "dl-progress") {

	    var db_eltchild = db_elt.getElementsByAttribute("class" , "db_progressMeter");
	    var db_eltmeter = db_eltchild[0];
	    var db_percentDone = db_dl.percentComplete;
	    
	    if (db_percentDone == -1) {
	      db_eltmeter.setAttribute("mode", "undetermined");
	    }
	    else {
	      db_eltmeter.setAttribute("value", db_percentDone);
	    }
	}
	if (topic == "dl-done") {
		
		//check if it's an image, if so, remove download after xx sec.? ADD PREF
		var db_fileext = elmpath.split(".").pop();
		if (db_fileext == "jpg" | db_fileext == "gif" | db_fileext == "jpeg" | db_fileext == "png" | db_fileext == "bmp") {
			var fixedelmpath = elmpath.replace(/\\/g, "\\\\");
			window.setTimeout("removeit('" + fixedelmpath + "','" + false + "')", 5000);
		}
		
	}
	
  }
}

function downbarinit() {
	
  window.removeEventListener("focus", downbarinit, false);
  
  //disable the downloadsidebar
  var sidebar = document.getElementById("sidebar-box");
  if (sidebar.getAttribute("sidebarcommand") == "viewDownloadsSidebar"){
  	toggleSidebar();  
  }
  document.getElementById("viewDownloadsSidebar").setAttribute("id", "xviewDownloadsSidebar");

  var downbarelem = document.getElementById("downbar");
  
  const db_dlmgrContractID = "@mozilla.org/download-manager;1";
  const db_dlmgrIID = Components.interfaces.nsIDownloadManager;
  db_gDownloadManager = Components.classes[db_dlmgrContractID].getService(db_dlmgrIID);

  var db_ds = db_gDownloadManager.datasource;

  downbarelem.database.AddDataSource(db_ds);
  downbarelem.builder.rebuild();
  
  //clear download history on start ADD PREF?
  //removeit(null, true);
  
  var db_observerService = Components.classes[db_kObserverServiceProgID]
                                  .getService(Components.interfaces.nsIObserverService);
  db_observerService.addObserver(db_dlObserver, "dl-progress", false);
  db_observerService.addObserver(db_dlObserver, "dl-done", false);
  //db_observerService.addObserver(db_dlObserver, "dl-cancel", false);
  //db_observerService.addObserver(db_dlObserver, "dl-failed", false);
  //db_observerService.addObserver(db_dlObserver, "dl-start", false);
}

function downbarclose() {
	
  var db_observerService = Components.classes[db_kObserverServiceProgID]
                                  .getService(Components.interfaces.nsIObserverService);
  db_observerService.removeObserver(db_dlObserver, "dl-progress");
  db_observerService.removeObserver(db_dlObserver, "dl-done");
  //db_observerService.removeObserver(db_dlObserver, "dl-cancel");
  //db_observerService.removeObserver(db_dlObserver, "dl-failed");
  //db_observerService.removeObserver(db_dlObserver, "dl-start");
}


function Commandswitcher(aCommand)  {

    switch (aCommand) {
    case "cmd_db_openfinished":
      openfinished(db_currentelem);
      break;
    case "cmd_db_showfile":
      showfile(db_currentelem);
      break;
    case "cmd_db_viewprop":
      viewprop(db_currentelem);
      break;
    case "cmd_db_remove":
      removeit(db_currentelem, false);
      break;
	case "cmd_db_cancel":
      cancelprogress(db_currentelem);
      break;
    case "cmd_db_expand":
      expandtowindow(db_currentelem);
      break;  
    default:
    }
}  

function openfinished(elemid) {
	var file = createLocalFile(elemid);
    file.launch();
    removeit(elemid);
}

function showfile(idtoshow) {
	var file = createLocalFile(idtoshow);
    file.reveal();
    removeit(idtoshow);
}

function viewprop(idtoview) {
	window.openDialog("chrome://browser/content/downloads/downloadProperties.xul",
                        "_blank", "modal,centerscreen,chrome,resizable=no", idtoview);                   

}

function removeit(idtoremove, allofthem){
	var downbarelem = document.getElementById("downbar");
	var observer = downbarelem.builder.QueryInterface(Components.interfaces.nsIRDFObserver);
    var ds = downbarelem.database;
    observer.beginUpdateBatch(ds);
	db_gDownloadManager.startBatchUpdate();
	try {
		if (allofthem == true) {
			var comparray = downbarelem.getElementsByAttribute("context","donecontext");
			for (i = 1; i <= comparray.length - 1; ++i) {
	        db_gDownloadManager.removeDownload(comparray[i].id);
	        }
		}
		else {
			db_gDownloadManager.removeDownload(idtoremove);
		}	
	}
	catch (e) {}
	observer.endUpdateBatch(ds);
	var remote = db_gDownloadManager.datasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remote.Flush();
    db_gDownloadManager.endBatchUpdate();
    downbarelem.builder.rebuild();
    
}

function cancelprogress(elemid) {
	db_gDownloadManager.cancelDownload(elemid);
}

function cancelall() {
	var downbarelem = document.getElementById("downbar");
	var inprogress = downbarelem.getElementsByAttribute("context", "progresscontext");
	for (i = 1; i <= inprogress.length - 1; ++i) {
        db_gDownloadManager.cancelDownload(inprogress[i].id);
        }
	
}

function expandtowindow (elemid) {
	
	/*This will be all that is needed in this function, pending on bug 200760
	 *db_gDownloadManager.openProgressDialogFor(elemid, window);
	 Ugly fix for now*/
	
	var pdialog = Components.classes["@mozilla.org/progressdialog;1"].createInstance(Components.interfaces.nsIProgressDialog);	
	var dl = pdialog.QueryInterface(Components.interfaces.nsIDownload);
	var adownload = Components.classes["@mozilla.org/download;1"].createInstance(Components.interfaces.nsIDownload);
	adownload = db_gDownloadManager.getDownload(elemid);
	//pdialog.cancelDownloadOnClose = false;  Not needed because the dialog is not connected to the download, the dialog is just listening
   
	dl.init(adownload.source, adownload.target, null, adownload.MIMEInfo, adownload.startTime, null);
  
	var dialistener = pdialog.QueryInterface(Components.interfaces.nsIWebProgressListener);
	adownload.listener = dialistener; 
	//internalDownload->SetDialog(pdialog); Can I do this in java? Need to access the nsdownload not the nsIDownload.
	//This would connect the download to the dialog, ie cancel download button would work on the dialog.
	pdialog.open(window);
	window.setTimeout(changecloselabel, 100);
	
	/* Attempt at altering the a progress dialog opened with the broken openProgressDialogFor, for now opening the window in this function instead
    //var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                   //.getService( Components.interfaces.nsIWindowWatcher );
	 //   aConsoleService.logStringMessage(ww.activeWindow.name);

	db_gDownloadManager.openProgressDialogFor(elemid, window);   
	*/
	
	/*old version 0.5
	//db_gDownloadManager.openProgressDialogFor(elemid, window);
	//document.getElementById(elemid).hidden = true;
	//access widow-mediator - dialog - CloseDownloadOnClose = false
	old*/
}

function updatecurrent(newelemid) {
	db_currentelem = newelemid;
}

function createLocalFile(aFilePath)
{
  var lfContractID = "@mozilla.org/file/local;1";
  var lfIID = Components.interfaces.nsILocalFile;
  var lf = Components.classes[lfContractID].createInstance(lfIID);
  lf.initWithPath(aFilePath);
  return lf;
}

function changecloselabel(){
	var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
	ww.activeWindow.document.getElementById("cancel").label = "Close"
}